<?php

namespace Drupal\last_visited_pages;

use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\Core\Controller\TitleResolverInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Database\Connection;
use Drupal\Component\Datetime\TimeInterface;

/**
 * Subscribes to the kernel request event to obliterate the default content.
 */
class LastVisitedPagesSubscriber implements EventSubscriberInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The route match service.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The title resolver service.
   *
   * @var \Drupal\Core\Controller\TitleResolverInterface
   */
  protected $titleResolver;

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The current path service.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPath;

  /**
   * The database connection service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * LastVisitedPagesSubscriber constructor.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user service.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match service.
   * @param \Drupal\Core\Controller\TitleResolverInterface $title_resolver
   *   The title resolver service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack service.
   * @param \Drupal\Core\Path\CurrentPathStack $current_path
   *   The current path service.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(
    AccountProxyInterface $current_user,
    RouteMatchInterface $route_match,
    TitleResolverInterface $title_resolver,
    RequestStack $request_stack,
    CurrentPathStack $current_path,
    Connection $database,
    TimeInterface $time,
  ) {
    $this->currentUser = $current_user;
    $this->routeMatch = $route_match;
    $this->titleResolver = $title_resolver;
    $this->requestStack = $request_stack;
    $this->currentPath = $current_path;
    $this->database = $database;
    $this->time = $time;
  }

  /**
   * Redirects the user when they're requesting our nearly blank page.
   *
   * @param \Symfony\Component\HttpKernel\Event\RequestEvent $event
   *   The request event.
   */
  public function checkForPlaces(RequestEvent $event) {
    // Grab the saved Last Visited Pages.
    // Use injected RequestStack.
    $request = $this->requestStack->getCurrentRequest();
    // Use injected RouteMatchInterface.
    $routeMatch = $this->routeMatch;
    $title = $this->titleResolver->getTitle($request, $routeMatch->getRouteObject());

    if (is_array($title)) {
      $title = array_key_exists('#markup', $title) ? $title['#markup'] : $title;
    }
    elseif (!is_null($title)) {
      $title = strip_tags($title);
    }

    // Save Last Visited Pages only for prominent destinations with titles.
    if (!is_null($title)) {
      $uid = $this->currentUser->id();
      $path = $this->currentPath->getPath();

      $this->database->insert('last_visited_pages')
        ->fields([
          'title',
          'path',
          'timestamp',
          'uid',
        ])
        ->values([
          $title,
          $path,
          $this->time->getRequestTime(),
          $uid,
        ])
        ->execute();

      $result_query_count = $this->database->select('last_visited_pages', 'n')
        ->fields('n', ['id'])
        ->condition('uid', $uid, '=')
        ->range(20, 100)
        ->orderBy('timestamp', 'DESC');

      $all_result_count = $result_query_count
        ->execute()
        ->fetchCol();

      if (!empty($all_result_count)) {
        $result_query_count = $this->database->delete('last_visited_pages')
          ->condition('id', $all_result_count, 'IN')
          ->execute();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // Subscribe to the kernel.request event.
    $events[KernelEvents::REQUEST][] = ['checkForPlaces'];
    return $events;
  }

}
