<?php

namespace Drupal\last_visited_pages\Plugin\Block;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Routing\RouteMatchInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Datetime\TimeInterface;

/**
 * Provides a Block to display last visited pages you visited on the website.
 *
 * @Block(
 *   id = "last_visited_pages_block",
 *   admin_label = @Translation("Last Visited Pages")
 * )
 */
class LastVisitedPagesBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * The database connection service.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $database;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Creates a LocalActionsBlock instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user service.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection service.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, RouteMatchInterface $route_match, AccountProxyInterface $current_user, Connection $database, DateFormatterInterface $date_formatter, ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager, TimeInterface $time) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->routeMatch = $route_match;
    $this->currentUser = $current_user;
    $this->database = $database;
    $this->dateFormatter = $date_formatter;
    $this->configFactory = $config_factory;
    $this->entityTypeManager = $entity_type_manager;
    $this->time = $time;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('current_route_match'),
      $container->get('current_user'),
      $container->get('database'),
      $container->get('date.formatter'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('datetime.time'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $this->configuration['num_places'] = 5;
    return ['label_display' => FALSE];
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $uid = $this->currentUser->id();

    // Grab the number of items to display.
    $num_items = $this->configuration['num_places'] ?: 5;
    $date_format = $this->configuration['date_format'] ?? 'medium';

    // Get Recently visited links.
    $result_query = $this->database->select('last_visited_pages', 'n')
      ->fields('n', [])
      ->condition('uid', $uid, '=')
      ->range(0, $num_items)
      ->orderBy('timestamp', 'DESC');

    $all_result = $result_query
      ->execute()
      ->fetchAllAssoc('id');

    $custom_format = FALSE;

    if ($date_format == 'custom') {
      $custom_format = TRUE;
      $date_format = $this->configuration['custom_date_format'] ?? 'medium';
    }

    // Initialize output variable.
    $last_visited_pages = '';

    if (!empty($all_result)) {
      foreach ($all_result as $item) {
        $url = Url::fromUri('base:' . $item->path, ['absolute' => TRUE])->toString();

        if ($item->title == '') {
          $item->title = 'Home';
        }

        if ($custom_format) {
          $last_visited_pages .= '<li><a href="' . $url . '">' . $item->title . '</a> | ' . $this->dateFormatter->format($item->timestamp, 'custom', $date_format) . ' </li>';
        }
        else {
          $last_visited_pages .= '<li><a href="' . $url . '">' . $item->title . '</a> | ' . $this->dateFormatter->format($item->timestamp, $date_format) . ' </li>';
        }
      }
    }

    // Check if we have any last visited pages to render.
    if (!empty($last_visited_pages)) {
      // Build the output.
      $output = '';

      // Display label if enabled.
      if ($this->configuration['label_display']) {
        $output .= '<h2>' . $this->label() . '</h2>';
      }

      // Wrap the last visited pages in a <ul> and return it as raw HTML.
      $output .= '<ul class="last-visited-pages">' . $last_visited_pages . '</ul>';

      // Return as raw markup using Markup::create().
      return ['#markup' => Markup::create($output)];
    }

    // If no pages to display, return empty output or an appropriate message.
    return ['#markup' => Markup::create('<p>' . $this->t('No recently visited pages.') . '</p>')];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    // Get the maximum allowed value from the configuration form.
    $max_places = $this->configFactory->get('last_visited_pages.settings')->get('max_places');

    $default_places = $this->configuration['num_places'] ?? 5;

    $form['label_display']['#default_value'] = $this->configuration['label_display'] ?? TRUE;

    // Add a select box of numbers form 1 to $max_to_display.
    $form['block_num_places_form'] = [
      '#type' => 'select',
      '#title' => $this->t('Number of items to show'),
      '#default_value' => $default_places,
      '#options' => array_combine(range(1, $max_places), range(1, $max_places)),
    ];

    $date_formats = [];

    $formats = $this->entityTypeManager
      ->getStorage('date_format')
      ->loadMultiple();

    foreach ($formats as $machine_name => $value) {
      $date_formats[$machine_name] = $this->t(
        '@name format: @date',
        [
          '@name' => $value->label(),
          '@date' => $this->dateFormatter->format($this->time->getRequestTime(), $machine_name),
        ]
      );
    }

    $date_formats['custom'] = $this->t('Custom');

    $form['date_format'] = [
      '#type' => 'select',
      '#title' => $this->t('Date format'),
      '#options' => $date_formats,
      '#default_value' => $this->configuration['date_format'] ?? 'medium',
    ];

    $form['custom_date_format'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Custom date format'),
      '#description' => $this->t('See <a href="https://www.php.net/manual/datetime.format.php#refsect1-datetime.format-parameters" target="_blank">the documentation for PHP date formats</a>.'),
      '#default_value' => $this->configuration['custom_date_format'] ?? '',
    ];

    $form['custom_date_format']['#states']['visible'][] = [
      ':input[name="settings[date_format]"]' => ['value' => 'custom'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['label_display'] = $form_state->getValue('label_display');
    $this->configuration['num_places'] = $form_state->getValue('block_num_places_form');
    $this->configuration['date_format'] = $form_state->getValue('date_format');
    $this->configuration['custom_date_format'] = $form_state->getValue('custom_date_format');
  }

  /**
   * {@inheritdoc}
   *
   * Disable block cache to keep it the Last Visited Pages update.
   */
  public function getCacheMaxAge() {
    return 0;
  }

}
